#!/usr/bin/php
<?php
define('PATH','/usr/local/esia/public_html/');
define('DRIVER_PATH','/usr/local/esia/public_html/plugin/virtualization/drivers/');

$_SERVER['DOCUMENT_ROOT']=PATH;
@include_once PATH.'esia.conf.php';
@include_once PATH.'plugin/virtualization/model_virtualization.php';

$dbconnection = new PDO('pgsql:host='.DB_HOST.';port='.DB_PORT.';dbname='.DB_NAME, DB_USERNAME, DB_PWD);
$dbconnection->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);

$STR_STATUS=array("OK","WARNING","CRITICAL","UNKNOWN");
$INT_STATUS=array_flip($STR_STATUS);
$PRIORITIES_STATES=array("CRITICAL","WARNING","UNKNOWN","OK");

function getParam($options,$name1,$name2,$default=null){
        if( isset( $options[$name1] ) ) return $options[$name1];
        if( isset( $options[$name2] ) ) return $options[$name2];
        return $default;
}

function exit_state($state,$msg=null,$usage=false){
        $STR_STATUS=$GLOBALS["STR_STATUS"];
        $INT_STATUS=$GLOBALS["INT_STATUS"];

        $ret=$state;
        if( is_string($state) ) $ret=$INT_STATUS[strtoupper($state)];
        else $state=$STR_STATUS[$ret];

        if( $msg != null ) echo "$state - $msg\n";
        if( $usage ) print_usage(true);
        exit($ret);
}

function calcul_state($a,$b=null){
        $STR_STATUS=$GLOBALS["STR_STATUS"];
        $INT_STATUS=$GLOBALS["INT_STATUS"];
        $PRIORITIES_STATES=$GLOBALS["PRIORITIES_STATES"];

        if(is_string($a) ) $a=$INT_STATUS[strtoupper($a)];
        if(is_string($b) ) $b=$INT_STATUS[strtoupper($b)];

        foreach($PRIORITIES_STATES as $value){
                $value = $INT_STATUS[$value];

                if( $value == $a || $value == $b ) return $value;
        }
        return $INT_STATUS["UNKNOWN"];
}

$SHORT_OPTS="H:";
$LONG_OPTS=array('host:');

$options = getopt($SHORT_OPTS, $LONG_OPTS);
$host = getParam($options,'H','host');

$cluster_node_available = FALSE;
$messages=array();

$all_state = $INT_STATUS["OK"];

try{
	$arch = Model_virtualization::getNodeInfraArch($dbconnection,$host);
	$cluster_node_available = TRUE;

	$messages[] = 'OK - Cluster node reacheable';
}
catch(VirtModelException $ex)
{
    exit_state('UNKNOWN',$ex->getMessage());
}
catch(Exception $ex)
{
	$all_state = calcul_state($INT_STATUS["WARNING"],$all_state);
	$messages[] = 'WARNING - '.trim($ex->getMessage());

    $driver = Model_virtualization::getAuthenticationParams($dbconnection,$host);
    if( $driver['driver'] == 'vmware' )
    {
        $all_state = calcul_state($INT_STATUS["CRITICAL"],$all_state);
        $messages[] = 'CRITICAL - VMware vcenter or standalone hypervisor unreacheable';
        echo 'Cluster GLOBAL STATUS is '.$STR_STATUS[$all_state]."\n";
        echo implode("\n",$messages);
        echo "\n";
        exit($all_state);
    }
}

if( !$cluster_node_available )
{
	$messages[] = 'Trying to found another reacheable cluster node';
	$cluster_conf = Model_virtualization::getAuthenticationParams($dbconnection,$host);
    $nodes = Model_virtualization::getAllClusterNodes($dbconnection,$host);
	$new_node_conf = FALSE;
	foreach($nodes as $node)
	{
		$node_conf = Model_virtualization::getAuthenticationParams($dbconnection,$node['device_id']);
		if( $node_conf['url'] != $cluster_conf['url'] )
		{
			try
			{
				$arch = Model_virtualization::getNodeInfraArch($dbconnection,$node['device_id']);
				$cluster_node_alive = TRUE;
                if( $new_node_conf == FALSE )
                {
				                $new_node_conf = $node_conf;
                }
				$messages[] = 'OK - Cluster node '.$node['device_name'] . ' ('.$node_conf['url'] .') available';
			}
			catch(Exception $ex){
                $all_state = calcul_state($INT_STATUS["WARNING"],$all_state);
				$messages[] = 'WARNING - Unable to access to cluster node '.$node['device_name'] . ' ('.$node_conf['url'] .') '.trim($ex->getMessage());
			}
		}
        else
        {
            $all_state = calcul_state($INT_STATUS["WARNING"],$all_state);
            $messages[] = 'WARNING - Unable to access to current conf cluster node '.$node['device_name'] . ' ('.$node_conf['url'] .') '.trim($ex->getMessage());
        }
	}
	if( $cluster_node_alive)
	{
		try
		{
			Model_virtualization::updateAuthenticationParams($dbconnection,$host,$new_node_conf['url'],$new_node_conf['login'],$new_node_conf['password']);
			$messages[] = 'OK - Cluster url updated with '.$new_node_conf['url'];
		}
		catch(Exception $ex)
		{
			$all_state = calcul_state($INT_STATUS["WARNING"],$all_state);
			$e_message = $ex->getMessage();
			if( strpos($e_message,'Permission denied') !== FALSE )
			{
				$e_message = 'Permission denied';
			}
			$messages[] = 'WARNING - Unable to update cluster url with '.$new_node_conf['url'].'. '.trim($e_message);
		}
	}
	else{
		$all_state = calcul_state($INT_STATUS["CRITICAL"],$all_state);
		$messages[] = 'CRITICAL - All cluster nodes are unreacheables';
	}
}

foreach($arch['hosts'] as $host)
{
	$node_status = 'DEAD';
	$esia_status = 'WARNING';
	if( $host['alive'] )
	{
		$esia_status = 'OK';
		$node_status = 'ALIVE';
	}
	$all_state = calcul_state($INT_STATUS[$esia_status],$all_state);
	$messages[] = sprintf('%s - Hypervisor %s is %s',$esia_status,$host['name'],$node_status);
}

echo 'Cluster GLOBAL STATUS is '.$STR_STATUS[$all_state]."\n";
echo implode("\n",$messages);
echo "\n";
exit($all_state);
