<?php
/**
    @file
    @brief Script permettant de récupérer des données et les afficher au format JSON
    @author Julien George
*/

$EXEC_PATH = '/usr/local/esia/public_html/plugin/virtcluster/drivers';
$RESOURCES = explode(',','cluster,host,storage,vm,template,volume,snapshot');

function exit_error($message){
    echo "ERROR : ";
    echo trim($message);
    echo "\n";
    exit(1);
}

//Vérification des paramètres
//Paramètre authfile
if( empty($argv[1] ) )  { exit_error("You must specify authfile file"); }
$o_authfilepath = $argv['1'];

if( stripos($o_authfilepath,'xen')){
    $drv_name = 'xenserver';
}
else if( stripos($o_authfilepath,'proxmox')){
    $drv_name = 'proxmox';
}
else { exit_error("Unable to identify driver name"); }

if(  !is_readable($o_authfilepath) )    { exit_error("Unable to access to authfile : $o_authfilepath"); }

//Paramètre resources
if( !isset($argv[2]) ){
    echo "ERROR : ";
    echo "You must specify a resource :\n";
    foreach($RESOURCES as $resource){
        echo "- $resource\n";
    }
    exit(1);
}

$o_resource = $argv[2];
if( !in_array($o_resource,$RESOURCES))  exit_error("Unknown resource type $o_resource. Resource must be one of this (".implode(',',$RESOURCES).').');

//Chargement du driveur
$drivers['proxmox']['file'] = 'proxmox/Proxmox.class.php';
$drivers['proxmox']['class'] = 'ProxmoxCluster';
$drivers['xenserver']['file'] = 'xenserver/XenServer.class.php';
$drivers['xenserver']['class'] = 'XenServerCluster';

$oldpath = get_include_path();
set_include_path($EXEC_PATH);
include_once($drivers[$drv_name]['file']);
set_include_path($oldpath);

$settings = file_get_contents($o_authfilepath);
$settings = json_decode($settings,TRUE);

if( $settings == FALSE )  { exit_error("Unable to parse json from authfile : $o_authfilepath"); }

$className = $drivers[$drv_name]['class'];
$clusterClass = new $className($settings);

$human_readeable = TRUE;

if( $o_resource == 'cluster' ){
    $data = $clusterClass->toArray(TRUE,FALSE,$human_readeable);
}
elseif( $o_resource == 'host'){
    foreach($clusterClass->getHosts() as $row){
	$data[]=$row->toArray(TRUE,FALSE,$human_readeable);
    }
}
elseif( $o_resource == 'vm'){
    foreach($clusterClass->getVMs() as $row){
	$data[]=$row->toArray(FALSE,FALSE,$human_readeable);
    }
}
elseif( $o_resource == 'template'){
    foreach($clusterClass->getTemplates() as $row){
	$data[]=$row;
    }
}
elseif( $o_resource == 'storage'){
    foreach($clusterClass->getStorages() as $row){
	$data[]=$row->toArray(FALSE,FALSE,$human_readeable);
    }
}

//print_r($data);
echo json_encode($data,JSON_PRETTY_PRINT)."\n";

exit(0);
