<?php
include_once "VMwareSoapClient.php";
//include_once "common.class.php";
//include_once "Device.class.php";
include_once "common/common.class.php";

class VMWareDriver
{
    protected $driver;
    protected $cache;

    public function __construct($settings)
    {
        $this->driver = new VMwareSoapClient($settings,TRUE);
        $this->cache = array();

    //    $data = $this->driver->getInventoryTree('VirtualMachine');
    //    print_r($data);
    //    exit(0);
    }

    public function cleanCache(){
        $this->cache = array();
    }

    public function getDriver()
    {
        return $this->driver;
    }

    public function listMetrics()
    {
        $counter_list = $this->getDriver()->getPerfsCounter();
        foreach($counter_list as $counter)
        {
            echo $counter['groupInfo']['key'];
            echo '.';
            echo $counter['nameInfo']['key'];
            echo '.';
            echo $counter['unitInfo']['key'];
            echo '.';
            echo $counter['rollupType'];
            echo " : ";
            echo $counter['nameInfo']['summary'];
            echo "\n";
        }
    }

    private function initData($ref,$object_type,$properties=NULL)
    {
        if( !isset( $this->cache["$object_type.data"][$ref]))
        {
            if( $properties == NULL )
            {
                $properties = '';
            }
            elseif( !is_array($properties))
            {
                $properties = explode(',',$properties);
            }
            $data = $this->driver->getView($object_type,$ref,$properties);
            $data = $data->returnval->propSet;

            if( $properties == '' )
            {
                $properties = array();
                foreach( $data as $row )
                {
                    $properties[] = $row->name;
                }
            }

            $row = array();
            $row['ref'] = $ref;
            $row['object_type'] = $object_type;
            foreach($data as $item)
            {
                $row[$item->name] = $item->val;
                //$row[str_replace('.','_',$item->name)] = $item->val;
            }
            $this->cache["$object_type.data"][$ref] = $row;
        }
    }

    private function initClusterList()
    {
        if( !isset($this->cache['ComputeResource.ref']))
        {
            $data = $this->driver->getInventoryTree('ComputeResource','name');
            $data = $data->returnval;
            if( !is_array($data) )
            {
                $data = array($data);
            }

            $isvcenter = FALSE;
            foreach($data as $row)
            {
                $type = $row->obj->type;
                $ref = $row->obj->{'_'};
                $name = $row->propSet->val;
                $this->cache['ComputeResource.ref'][$name]=$ref;

                if( !$isvcenter && $type == 'ClusterComputeResource' )
                {
                     $isvcenter = TRUE;
                }
            }

            $this->cache['val.isVcenter'] = $isvcenter;
        }
    }

    private function initHostList($cluster_ref)
    {
        if( !isset( $this->cache['HostSystem.ref'][$cluster_ref]))
        {
            $data = $this->driver->getView('ComputeResource',$cluster_ref,'host');
            $data = $data->returnval->propSet->val->ManagedObjectReference;
            if( !is_array($data) )
            {
                $data = array($data);
            }
            foreach($data as $row)
            {
                $this->cache['HostSystem.ref'][$cluster_ref][] = $row->{'_'};
            }
        }
    }

    private function initDatastoreList($cluster_ref)
    {
        if( !isset( $this->cache['Datastore.ref'][$cluster_ref]))
        {
            $data = $this->driver->getView('ComputeResource',$cluster_ref,'datastore');
            $data = $data->returnval->propSet->val->ManagedObjectReference;
            if( !is_array($data) )
            {
                $data = array($data);
            }
            foreach($data as $row)
            {
                $this->cache['Datastore.ref'][$cluster_ref][] = $row->{'_'};
            }
        }
    }

    private function initVMList($host_Ref)
    {
        if( !isset( $this->cache['VirtualMachine.ref'][$host_Ref]))
        {
            $data = $this->driver->getView('HostSystem',$host_Ref,'vm');
            $data = $data->returnval->propSet->val->ManagedObjectReference;
            if( !is_array($data) )
            {
                $data = array($data);
            }
            foreach($data as $row)
            {
                $this->cache['VirtualMachine.ref'][$host_Ref][] = $row->{'_'};
            }
        }
    }

    public function getClusterListRef()
    {
        $this->initClusterList();
        return $this->cache['ComputeResource.ref'];
    }

    public function getClusterListName()
    {
        $this->initClusterList();
        return array_keys($this->cache['ComputeResource.ref']);
    }

    public function getClusterRefByName($cluster_name)
    {
        $this->initClusterList();
        return $this->cache['ComputeResource.ref'][$cluster_name];
    }

    public function getDatastoreListRef($cluster_ref)
    {
        $this->initDatastoreList($cluster_ref);
        return $this->cache['Datastore.ref'][$cluster_ref];
    }

    public function getHostListRef($cluster_ref)
    {
        $this->initHostList($cluster_ref);
        return $this->cache['HostSystem.ref'][$cluster_ref];
    }

    public function getVMListRef($host_ref)
    {
        $this->initVMList($host_ref);
        return $this->cache['VirtualMachine.ref'][$host_ref];
    }

    public function getClusterData($cluster_ref)
    {
        $properties = 'name,summary.numHosts';
        if( $this->isVCenter($cluster_ref))
        {
            $this->initData($cluster_ref,'ClusterComputeResource',"$properties,configuration.dasConfig.enabled,configuration.dasVmConfig");
            return $this->cache['ClusterComputeResource.data'][$cluster_ref];
        }
        else
        {
            $this->initData($cluster_ref,'ComputeResource',$properties);
            return $this->cache['ComputeResource.data'][$cluster_ref];
        }
    }

    public function getStorageData($storage_ref)
    {
        $properties_fields='info,summary';
        $properties_other='vm,host';
        $this->initData($storage_ref,'Datastore',"$properties_fields,$properties_other");
        return $this->cache['Datastore.data'][$storage_ref];
    }

    public function getHostData($host_ref)
    {
        $cluster = 'name,summary.managementServerIp';
        $host_construct = 'summary.config,summary.hardware,config.network.vnic';
        $host_fields = 'runtime,summary.quickStats';
        $host_other = 'config.network.vswitch,config.network.pnic';
        $this->initData($host_ref,'HostSystem',"$cluster,$host_construct,$host_fields,$host_other");
        return $this->cache['HostSystem.data'][$host_ref];
    }

    public function getVirtualMachineData($vm_ref)
    {
        $construct = 'name,guest,config';
        $fields = 'runtime,summary.quickStats,summary.guest';
        $other = 'config.hardware,summary.config,recentTask,layoutEx,layout';

        $this->initData($vm_ref,'VirtualMachine',"$construct,$fields,$other");
        return $this->cache['VirtualMachine.data'][$vm_ref];
    }

    //La machine interrogée est-t'elle le Vcenter ?
    public function isVCenter()
    {
        $this->initClusterList();
        return $this->cache['val.isVcenter'];
    }

    //Y a t'il un Vcenter quelquepart ?
    public function isVcenterAvailable($cluster_ref)
    {
        if( $this->isVCenter() )
        {
            return TRUE;
        }
        $host_list = $this->getHostListRef($cluster_ref);
        $host_data = $this->getHostData(array_shift($host_list));
        if( !empty($host_data['managementServerIp']) && filter_var($host_data['managementServerIp'], FILTER_VALIDATE_IP))
        {
            return TRUE;
        }
        return FALSE;
    }

    public function getPerfs($type,$ref,$keys,$period=120,$instance=NULL)
    {
        $cache_key = "$type|$ref|$keys|$period|$instance";
        $interval = 20;
        $samples = $period/$interval;

        $driver = $this->driver;

        if( !isset($this->cache['Counter.data'][$cache_key]))
        {
            if( !isset($this->cache['Counter.keys']) )
            {
                $this->cache['Counter.keys'] = array();
                $this->cache['Counter.data'] = array();
            }

            if(!isset($this->cache['Counter.keys'][$keys]))
            {
                $exp = explode('.',$keys);
                $counter = $driver->getPerfsCounter($exp[0],$exp[1],$exp[2],$exp[3]);
                $this->cache['Counter.keys'][$keys] = array_shift($counter);
            }

            $counter = $this->cache['Counter.keys'][$keys];
            $metric = $counter['key'];

            if( $instance !== NULL )
            {
                $metric = array('counterId'=>$metric,'instance'=>$instance);
            }

            $this->cache['Counter.data'][$cache_key] = $driver->getPerfsData($type,$ref,$metric,$samples,$interval);
        }
        $data = $this->cache['Counter.data'][$cache_key];
        return $data->returnval->value;
    }
}

class VMwareCluster extends Cluster{
    protected $driver;
    protected $ref;
    protected $name;
    protected $cache=NULL;

    protected $vm_ha = NULL;

    public function __construct($settings)
    {
        $url = $settings['url'];
        $settings['url'] = explode('?',$settings['url'])[0];
        $this->driver = new VMWareDriver($settings);

        $query = parse_url($url, PHP_URL_QUERY);
        $output = array();
        if( $query )
        {
            parse_str($query, $output);
        }

        if( !$this->isVCenter() || count($this->getClusterList()) == 1 )
        {
            $cluster_list = $this->getClusterList();
            $name = array_pop($cluster_list);
            $this->setClusterName($name);
        }
        elseif( isset($output['cluster']))
        {
            $this->setClusterName($output['cluster']);
        }
        elseif( isset($settings['cluster'] ) )
        {
            $this->setClusterName($settings['cluster']);
        }
    }

    public function getData($type,$ref,$key=NULL)
    {
        if( $type == 'Datastore' )
        {
            $data = $this->driver->getStorageData($ref);
        }
        elseif($type == 'HostSystem' )
        {
            $data = $this->driver->getHostData($ref);
        }
        elseif($type == 'VirtualMachine' )
        {
            $data = $this->driver->getVirtualMachineData($ref);
        }
        else
        {
            return null;
        }
        if( $key == NULL )
        {
            return $data[$key];
        }
        else
        {
            return $data[$key];
        }
    }

    public function getClusterList()
    {
        return $this->driver->getClusterListName();
    }

    public function setClusterName($name)
    {
        $this->name = $name;
        $this->ref = $this->driver->getClusterRefByName($name);
    }

    public function getClusterRef()
    {
        return $this->ref;
    }

    public function getPerfs($type,$ref,$keys,$period=120,$instance=NULL)
    {
        return $this->getDriver()->getPerfs($type,$ref,$keys,$period,$instance);
    }

    public function getKBpsMetric($obj_type,$obj_ref,$metric,$iface=NULL)
    {
        $period = 120;
        $data = $this->getPerfs($obj_type,$obj_ref,"$metric.kiloBytesPerSecond.average",$period);
        if( $iface == 'global' )
        {
            $iface = NULL;
        }

        if( is_object($data) )
        {
            $data = array($data);
        }
        foreach($data as $row)
        {
            if (isset($row->id->instance))
            {
                //Le global à un nom d'instance vide
                if( $iface == NULL && empty($row->id->instance))
                {
                    $row = explode(',',$row->value);
                    $row = array_sum($row)/count($row);
                    return round($row*1024);
                }
                else if ($iface == $row->id->instance)
                {
                    $row = explode(',',$row->value);
                    $row = array_sum($row)/count($row);
                    return round($row*1024);
                }
            }
        }
        return NULL;
    }

    public function isVcenter()
    {
        return $this->driver->isVCenter($this->ref);
    }

    public function isVcenterAvailable()
    {
        if( $this->isVcenter() )
        {
            return TRUE;
        }
        return $this->driver->isVcenterAvailable($this->ref);
    }

    public function cleanCache(){
        $this->driver->cleanCache();
    }

    public function jsonSerialize(){
        $cluster_status['name'] = $this->getName();
        $cluster_status['master'] = $this->getMasterHost();
        $cluster_status['is_cluster'] = $this->isCluster();
        $cluster_status['is_ha'] = $this->isHa();
        return $cluster_status;
    }

    public function getDriver(){
        return $this->driver;
    }

    public function getName(){
        return $this->name;
    }

    public function getMasterHost(){
        if( !$this->isVcenter() )
        {
            return NULL;
        }
        $master = NULL;
        foreach($this->getHosts() as $host)
        {
            if( $host->isMaster() === TRUE)
            {
                $master = $host->getName();
                break;
            }
        }
        return $master;
    }

    public function isCluster(){
        $data = $this->driver->getClusterData($this->ref);
        return ( $data['summary.numHosts'] > 1 );
    }

    public function isHA()
    {
        if( !$this->isVcenter() )
        {
            return NULL;
        }
        $data = $this->driver->getClusterData($this->ref);
        return ( isset($data['configuration.dasConfig.enabled']) && $data['configuration.dasConfig.enabled']);
    }

    public function getVMHA($vm_ref)
    {
        if( $this->isHA() === TRUE )
        {
            if( $this->vm_ha == NULL)
            {
                $this->vm_ha = array();
                $data = $this->driver->getClusterData($this->ref);
                $data = $data['configuration.dasVmConfig']->ClusterDasVmConfigInfo;
                foreach($data as $vmconf)
                {
                    $this->vm_ha[$vmconf->key->{'_'}] = $vmconf->dasSettings;
                }
            }
            if( isset($this->vm_ha[$vm_ref]))
            {
                return $this->vm_ha[$vm_ref];
            }
            return FALSE;
        }
        else
        {
            return $this->isHA();
        }
    }

    public function getHosts():array{
        $host_list = $this->driver->getHostListRef($this->ref);
        $ret = array();
        foreach($host_list as $ref)
        {
            $ret[] = new VMwareHost($ref,$this);
        }
        return $ret;
    }

    public function getHostByName($name){
        foreach($this->getHosts() as $host){
            if( $host->getName() == $name)
            {
                return $host;
            }
        }
    }

    public function getVMs():array{
        $vm_list = array();
        foreach($this->getHosts() as $host){
            $vm_list = array_merge($vm_list,$host->getVMs());
        }
        return $vm_list;
    }

    public function getTemplates():array{
        $vm_list = array();
        foreach($this->getHosts() as $host){
            $vm_list = array_merge($vm_list,$host->getTemplates());
        }
        return $vm_list;
    }

    public function getStorages($shared=NULL):array{
        $storage_list = $this->driver->getDatastoreListRef($this->ref);
        $ret = array();
        foreach($storage_list as $ref)
        {
            $storage = new VMwareStorage($ref,$this);
            if( $shared === NULL || $shared == $storage->isShared() )
            {
                $ret[] = $storage;
            }
        }
        return $ret;
    }

    public function getVMFromSnapshot($snapid){
        $snap_data = $this->driver->getDriver()->getView('VirtualMachineSnapshot',$snapid,'vm');
        if( isset( $snap_data->returnval->propSet->val->{'_'} ) );
        {
            $vm_ref = $snap_data->returnval->propSet->val->{'_'};
            return $this->getVM($vm_ref);
        }
        return NULL;
    }

    function getVMByName($name){
        foreach($this->getHosts() as $host){
            $vm_list = $host->getVMs();
            foreach($vm_list as $vm)
            {
                if( $name == $vm->getName())
                {
                    return array($vm);
                }
            }
        }
    }

    public function getVM($vmid,$refresh=FALSE){
        /*
        foreach($this->getHosts() as $host){
            $vm_list = $this->getDriver()->getVMListRef($host->getID());
            foreach($vm_list as $ref)
            {
                if( $vmid == $ref)
                {
                    return new VMwareVM($ref,$host);
                }
            }
        }
        */
        if( $refresh )
        {
            $this->cleanCache();
        }
	try
	{
		$host_data = $this->driver->getDriver()->getView('VirtualMachine',$vmid,'runtime.host');
		if( isset($host_data->returnval->propSet->val->{'_'}))
		{
			$host_ref = $host_data->returnval->propSet->val->{'_'};
			return new VMwareVM($vmid,new VMwareHost($host_ref,$this));
		}
	}
	catch(Exception $ex){}
        return NULL;
    }

    public function cloneVM($source,$target_name,$full=TRUE){
        return NULL;
    }
}

class VMwareStorage extends Storage{
    protected $cluster;
    protected $ref;

    public function getData($key)
    {
        return $this->cluster->getData('Datastore',$this->ref,$key);
    }

    public function getPerfs($type,$ref,$keys,$period=120,$instance=NULL)
    {
        return $this->cluster->getPerfs($type,$ref,$keys,$period,$instance);
    }

    public function __construct($ref,VMwareCluster $cluster){
        $this->cluster = $cluster;
        $this->ref = $ref;
    }

    public function getId()
    {
        return $this->ref;
    }

    public function getName()
    {
        $data = $this->getData('summary');
        return $data->name;
    }

    public function isShared()
    {
        $data = $this->getData('summary');
        if( !isset($data->multipleHostAccess))
        {
            return NULL;
        }
        return ( $data->multipleHostAccess == 1 );
    }

    public function getType()
    {
        $data = $this->getData('summary');
        return $data->type;
    }

    public function getSizeTotal(){
        $data = $this->getData('summary');
        return $data->capacity;
    }

    public function getSizeUsed(){
        $data = $this->getData('summary');
        return ( $data->capacity - $data->freeSpace );
    }

    public function getSizeVirtual(){
        $data = $this->getData('info');
        return $data->maxVirtualDiskCapacity;
    }

    public function getHosts()
    {
        $data = $this->getData('host');
        $hosts = array();

        if( isset($data->DatastoreHostMount->key) )
        {
            $data->DatastoreHostMount = array($data->DatastoreHostMount);
        }

        foreach($data->DatastoreHostMount as $item)
        {
            $hosts[] = new VMwareHost($item->key->{'_'},$this->cluster);
        }
        return $hosts;
    }

    public function getVolumes()
    {
        $data = $this->getData('vm');

        if( !isset($data->ManagedObjectReference))
        {
            return array();
        }
        if( !is_array($data->ManagedObjectReference))
        {
            $data->ManagedObjectReference = array($data->ManagedObjectReference);
        }

        $ret_volumes = array();
        foreach($data->ManagedObjectReference as $vm)
        {
            $vm_ref = $vm->{'_'};
            $vm = $this->cluster->getVM($vm_ref);
            $vm_volumes = $vm->getVolumes();

            foreach($vm_volumes as $vo)
            {
		$sr = $vo['storage'];
		if( $sr->getId() == $this->ref )
		{
                        $ret_volumes[] = $vo;
		}
	    }
        }
        return $ret_volumes;
    }
}


class VMwareHost extends Host{
    protected $cluster;
    protected $ref;

    public function getData($key)
    {
        return $this->cluster->getData('HostSystem',$this->ref,$key);
    }

    public function getPerfs($type,$ref,$keys,$period=120,$instance=NULL)
    {
        return $this->cluster->getPerfs($type,$ref,$keys,$period,$instance);
    }

    public function __construct($ref,VMwareCluster $cluster){
        $this->cluster = $cluster;
        $this->ref = $ref;

        $host_data = $this->cluster->getDriver()->getHostData($ref);

        $host_net = $host_data['config.network.vnic'];
        $host_conf = $host_data['summary.config'];
        $host_hw = $host_data['summary.hardware'];

        $this->setId($ref);
        $this->setName($host_data['name']);
        $vendor = $host_hw->vendor;
        $model = $host_hw->model;

        $identifierValue = '';
        foreach($host_hw->otherIdentifyingInfo as $identifier )
        {
            if( $identifier->identifierType->key == 'EnclosureSerialNumberTag' )
            {
                $identifierValue = $identifier->identifierValue;
            }
        }

        $this->setDescription("$vendor $model $identifierValue");
        $this->setType('hypervisor');

        $this->setManufacturer($host_conf->product->vendor);
        $this->setOS($host_conf->product->name);
        $this->setOSVersion($host_conf->product->version);

        if( is_array($host_net) )
        {
                $host_net = array_shift($host_net);
        }
        $hostVirtualNic = $host_net->HostVirtualNic;
        if( is_array($hostVirtualNic) )
        {
                $hostVirtualNic = array_shift($hostVirtualNic);
        }

        $this->setAddress($hostVirtualNic->spec->ip->ipAddress);
    }

    public function isMaster()
    {
        $data = $this->getData('runtime');
        if(! isset($data->dasHostState))
        {
            return NULL;
        }
        elseif( $data->dasHostState->state == 'connectedToMaster' )
        {
            return FALSE;
        }
        elseif( $data->dasHostState->state == 'master' )
        {
            return TRUE;
        }
        else return NULL;
    }

    public function isAlive()
    {
        $runtime = $this->getData('runtime');
        return ($runtime->connectionState=='connected' && $runtime->powerState=='poweredOn' && $runtime->inMaintenanceMode==FALSE);
    }

    public function getUpTime()
    {
        $data = $this->getData('summary.quickStats');
        return $data->uptime;
    }

    public function getCpusTotal()
    {
        $data = $this->getData('summary.hardware');
        return $data->numCpuCores;
    }

    public function getCpusUsage()
    {
        $period = 120;
        $usage = $this->getPerfs('HostSystem',$this->ref,'cpu.usage.percent.average',$period,'');
        $usage = $usage->value;
        $usage = explode(',',$usage);
        $usage = array_sum($usage)/count($usage);
        //exit(0);
        return ( $usage * 0.01 * 0.01);

        /*
        $usagemhz = $this->getPerfs('HostSystem','cpu.usagemhz.megaHertz.average',$period);
        $usagemhz = $usagemhz->value;
        $usagemhz = explode(',',$usagemhz);
        $usagemhz = array_sum($usagemhz)/count($usagemhz);

        $hardware = $this->getData('host.hardware',TRUE,'HostSystem','summary.hardware');

        $numCpuPkgs = $hardware->val->numCpuPkgs;
        $numCpuCores = $hardware->val->numCpuCores;
        $cpuMhz = $hardware->val->cpuMhz;

        $totalMhz = $numCpuPkgs * $numCpuCores * $cpuMhz;
        return ( $usagemhz / $totalMhz );
        */
    }

    public function getMemTotal()
    {
        $data = $this->getData('summary.hardware');
        return $data->memorySize;
    }

    public function getMemUsage()
    {
        $data = $this->getData('summary.quickStats');
        return ( $data->overallMemoryUsage * 1024 * 1024);
    }

    private function getIfaceMetric($metric,$iface=NULL)
    {
        return $this->getCluster()->getKBpsMetric('HostSystem',$this->ref,'net.'.$metric,$iface);
    }

    public function getNetInUsage(){
        return $this->getIfaceMetric('bytesRx');
    }

    public function getNetOutUsage(){
        return $this->getIfaceMetric('bytesTx');
    }

    public function getNetInterfaces(){
        $pif_speed = array();
        $pif_speed['global']['rx'] = $this->getNetInUsage();
        $pif_speed['global']['tx'] = $this->getNetOutUsage();

        $data = $this->getData('config.network.vswitch');
        $mtu_list = array();
        foreach($data->HostVirtualSwitch as $vswitch)
        {
            foreach($vswitch->spec->bridge->nicDevice as $pif)
            {
                $mtu_list[$pif] = $vswitch->mtu;
            }
        }

        $data = $this->getData('config.network.pnic');
        foreach($data->PhysicalNic as $pif)
        {
            $retRow = array();
            $retRow['ip'] = $pif->spec->ip->ipAddress;
            $retRow['mac'] = $pif->mac;
            if(isset($mtu_list[$pif->device]))
            {
                $retRow['mtu'] = $mtu_list[$pif->device];
            }
            $retRow['rx'] = $this->getIfaceMetric('bytesRx',$pif->device);
            $retRow['tx'] = $this->getIfaceMetric('bytesTx',$pif->device);
            if( isset($pif->linkSpeed->speedMb))
            {
                $retRow['speed'] = $pif->linkSpeed->speedMb;
            }
            $pif_speed[$pif->device] = $retRow;
        }
        return $pif_speed;
    }

    public function getVMs()
    {
        $vm_list = $this->cluster->getDriver()->getVMListRef($this->ref);

        $ret = array();
        foreach($vm_list as $ref)
        {
            $vm = new VMwareVM($ref,$this);
            if( !$vm->isTemplate() )
            {
                $ret[] =$vm;
            }
        }
        return $ret;
    }

    public function getTemplates()
    {
        $vm_list = $this->cluster->getDriver()->getVMListRef($this->ref);

        $ret = array();
        foreach($vm_list as $ref)
        {
            $vm = new VMwareVM($ref,$this);
            if( $vm->isTemplate() )
            {
                $ret[] = $vm->toTemplateArray();
            }
        }
        return $ret;
    }

    public function getCluster(){
        return $this->cluster;
    }
/*
    public function getDiskRead()
    {
        $period = 120;
        $usage = $this->getPerfs('Datastore',$this->ref,'disk.read.kiloBytesPerSecond.average',$period,'*');
        print_r($usage);
    }
    */
}

class VMwareVM extends VM
{
    protected $host;
    protected $cluster;
    protected $ref;

    protected $ha_settings = NULL;

    private function getIfaceMetric($metric,$iface=NULL)
    {
        if( !$this->isAlive() )
        {
            return 0;
        }
        return $this->getCluster()->getKBpsMetric('VirtualMachine',$this->ref,'net.'.$metric,$iface);
    }

    private function getDiskMetric($metric,$disk=NULL)
    {
        if( !$this->isAlive() )
        {
            return 0;
        }
        $data = $this->getCluster()->getKBpsMetric('VirtualMachine',$this->ref,'virtualDisk.'.$metric,$disk);
        return $data;
    }

    private static $enum_power_states = array(
        'poweredon' => 'running',
        'poweredoff' => 'stopped',
        'suspended'  => 'paused',
    );

    public function getPerfs($ref,$keys,$period=120,$instance=NULL)
    {
        return $this->cluster->getPerfs('VirtualMachine',$ref,$keys,$period,$instance);
    }

    public function getCluster(){
        return $this->cluster;
    }

    public function getData($key)
    {
        return $this->cluster->getData('VirtualMachine',$this->ref,$key);
    }

    public function __construct($ref,VMwareHost $host){
        $this->host = $host;
        $this->cluster = $host->getCluster();
        $this->ref = $ref;

        $driver = $this->cluster->getDriver();

        $vm_data = $this->cluster->getDriver()->getVirtualMachineData($ref);

        $vm_config = $vm_data['config'];
        $vm_guest = $vm_data['guest'];

        $this->setId($ref);
        $this->setName($vm_data['name']);
        $this->setType('VM');
        $this->setManufacturer($host->getManufacturer());

        if( isset($vm_config->annotation) )
        {
            $this->setDescription($vm_config->annotation);
        }
        if( isset($vm_guest->guestFamily) )
        {
            $this->setOS($vm_guest->guestFamily);
        }
        if( isset($vm_guest->guestFullName) )
        {
            $this->setOSVersion($vm_guest->guestFullName);
        }
        if( isset($vm_guest->ipAddress) )
        {
            $this->setAddress($vm_guest->ipAddress);
        }
    }

    public function getHost(){
        return $this->host;
    }

    public function getState(){
        $data = $this->getData('runtime');
        return self::$enum_power_states[strtolower($data->powerState)];
    }

    public function isLocked($refresh=FALSE){
        if( $refresh )
        {
            $this->cluster->cleanCache();
        }
        $data = $this->getLockStatus();
        return ( $data !== NULL );
    }

    public function getLockStatus($refresh=FALSE){
        if( $refresh )
        {
            $this->cluster->cleanCache();
        }
        $data = $this->getData('recentTask');
        if( empty($data->ManagedObjectReference) )
        {
            return NULL;
        }
        else
        {
            $data = $this->cluster->getDriver()->getDriver()->getView('Task',$data->ManagedObjectReference->{'_'});
            if( !isset($data->returnval->propSet) )
            {
                return json_encode($data);
            }

            foreach($data->returnval->propSet as $prop)
            {
                if( $prop->name == 'info' )
                {
                    if( isset($prop->val->completeTime))
                    {
                        return NULL;
                    }
                    elseif(!isset($prop->val->descriptionId))
                    {
                        return json_encode($prop->val);
                    }
                    else
                    {
                        return $prop->val->descriptionId;
                    }
                }
            }
            return json_encode($data->returnval->propSet);
        }
        return json_encode($data);
    }

    public function isAlive(){
        return ( $this->getState() == 'running' );
    }

    public function getUpTime(){
        if( !$this->isAlive() )
        {
            return NULL;
        }
        $data = $this->getData('summary.quickStats');
        return $data->uptimeSeconds;
    }

    public function getCpuPlatform(){
        $data = $this->getData('runtime');
        if( !isset($data->offlineFeatureRequirement) || !isset($data->offlineFeatureRequirement->featureName) )
        {
            return NULL;
        }
        return $data->offlineFeatureRequirement->featureName;
    }

    public function isAgent(){
        $data = $this->getData('summary.guest');
        return ( $data->toolsStatus == 'toolsOk' && $data->toolsRunningStatus == 'guestToolsRunning' );
    }

    public function getCpusTotal(){
        $data = $this->getData('config.hardware');
        $cpus = $data->numCPU;
        $cores = $data->numCoresPerSocket;
        return $cpus*$cores;
    }

    public function getCpusUsage(){
        if( !$this->isAlive() )
        {
            return 0;
        }
        $period = 120;
        $usage = $this->getPerfs($this->ref,'cpu.usage.percent.average',$period,'');
        $usage = $usage->value;
        $usage = explode(',',$usage);
        $usage = array_sum($usage)/count($usage);
        return ( $usage * 0.01 * 0.01);
    }

    public function getMemTotal(){
        $data = $this->getData('summary.config');
        return ( $data->memorySizeMB * 1024 * 1024 );
    }

    public function getMemUsage(){
        $data = $this->getData('summary.quickStats');
        return ( $data->guestMemoryUsage * 1024 * 1024 );
    }

    public function getNetInUsage(){
        return $this->getIfaceMetric('bytesRx');
    }

    public function getNetOutUsage(){
        return $this->getIfaceMetric('bytesTx');
    }

    public function getDiskRead(){
        return $this->getDiskMetric('read');
    }

    public function getDiskWrite(){
        return $this->getDiskMetric('write');
    }

    public function getHASettings(){
        if( $this->isHA() === TRUE )
        {
            return json_encode($this->cluster->getVMHA($this->ref));
        }
    }

    public function isHA(){
        $settings = $this->cluster->getVMHA($this->ref);
        if( $settings !== FALSE && $settings !== NULL )
        {
            return TRUE;
        }
        else
        {
            return $settings;
        }
    }

    public function isTemplate()
    {
        $data = $this->getData('config');
        return $data->template;
    }

    public function getSnapshots(){
        $data = $this->getData('layout');
        if( !isset($data->snapshot))
        {
            return array();
        }

        if( !is_array($data->snapshot) )
        {
            $data->snapshot = array($data->snapshot);
        }
        $ret = array();
        foreach($data->snapshot as $snap)
        {
            $snap_data = $this->cluster->getDriver()->getDriver()->getView('VirtualMachineSnapshot',$snap->key->{'_'},'config');
            $snap_data = $snap_data->returnval->propSet;

            if(  $snap_data->name == 'config' )
            {
                $snap_data = $snap_data->val;

                $retRow = array();
                $retRow['id'] = $snap->key->{'_'};
                $retRow['name'] = $snap_data->name;
		if( isset( $snap_data->annotation))
		{
			$retRow['description'] = $snap_data->annotation;
		}
		else
		{
			$retRow['description'] = NULL;
		}
                $retRow['parent'] = NULL;
                $retRow['time'] = strtotime($snap_data->modified);
                $retRow['withRAM']= NULL;
                $ret[] = $retRow;
            }
        }
        return $ret;
    }

    public function getVolumes(){
        //On récupére la list des instances des disque de cette VMs pour récupérer plus tard les read/write bps
        $instances = array();
        if( $this->isAlive())
        {
            $metrics = $this->getCluster()->getPerfs('VirtualMachine',$this->ref,"virtualDisk.read.kiloBytesPerSecond.average",120);
            foreach($metrics as $row)
            {
                if( !empty($row->id->instance))
                {
                    $exp = explode(':',$row->id->instance);
                    $instances[array_pop($exp)] = $row->id->instance;
                }
            }
        }

        //Calcul des tailles des disques
        $data = $this->getData('layoutEx');
        $disk_usage = array();

        if( isset($data->disk->key) )
        {
            $data->disk = array($data->disk);
        }

        foreach( $data->disk as $disk)
        {
            $disk_usage[$disk->key] = 0;

            //Si il y a un snapshot, on peut se retrouver avec un tableau, on prend alors le dernier disque
            if(!isset($disk->chain->fileKey))
            {
                $disk->chain = array_pop($disk->chain);
            }

            foreach( $disk->chain->fileKey as $file_idx)
            {
                $disk_usage[$disk->key]+=$data->file[$file_idx]->size;
            }
        }

        //Création de l'array volume
        $data = $this->getData('config.hardware');
        $volumes = array();
        $device_keys = array();
        foreach($data->device as $hardware)
        {
            if( isset( $hardware->backing->datastore ) )
            {
                if( isset( $hardware->diskObjectId ) )
                {
                    $row = array();
                    $row['id'] = $hardware->diskObjectId;
                    $device_id = $hardware->key;
                    $row['name'] = $hardware->deviceInfo->label;
                    $row['desc'] = $hardware->backing->fileName;
                    $row['size_used'] = $disk_usage[$device_id];
                    $row['size_virt'] = $hardware->capacityInBytes;
                    if( isset($hardware->backing->parent) )
                    {
                        $row['parent'] = $hardware->backing->parent->fileName;
                    }
                    else
                    {
                        $row['parent'] = NULL;
                    }
                    $row['storage'] = new VMwareStorage($hardware->backing->datastore->{'_'},$this->cluster);
                    $row['vm'] = array($this);
                    $row['io_read'] = 0;
                    $row['io_write'] = 0;

                    if( !empty($instances))
                    {
                        $row['io_read'] = $this->getDiskMetric('read',$instances[$hardware->unitNumber]);
                        $row['io_write'] = $this->getDiskMetric('write',$instances[$hardware->unitNumber]);
                    }

                    $device_keys[] = $device_id;
                    $volumes[] = $row;
                }
            }
        }
        return $volumes;
    }

    public function start()
    {
        $driver = $this->getCluster()->getDriver();
        $driver->PowerOnVM($this->getID(),$this->getHost()->getID());
/*
        $ref = $this->cluster->getRefByUuid('VM',$vmid);

        if( $state == 'suspended' ){
            $data = $this->cluster->getDriver()->get('Async.VM.resume',array($ref,FALSE,FALSE));
        }
        elseif( $state == 'paused' ){
            $data = $this->cluster->getDriver()->get('Async.VM.unpause',array($ref));
        }
        else{
            $data = $this->cluster->getDriver()->get('Async.VM.start',array($ref,FALSE,FALSE));
        }
        */
    }

    public function shutdown($cleanly=TRUE){
        /*
        $allowed = $this->isAllowedOperation('shutdown',$cleanly);
        if( !$allowed['allowed'] ) throw new Exception($allowed['cause']);

        $vmid = $this->getId();
        $ref = $this->cluster->getRefByUuid('VM',$vmid);

        if( !$cleanly ){
            $data = $this->cluster->getDriver()->get('Async.VM.hard_shutdown',array($ref));
        }
        else{
            $data = $this->cluster->getDriver()->get('Async.VM.clean_shutdown',array($ref));
        }
        */
    }

    public function reboot($cleanly=TRUE){
        /*
        $allowed = $this->isAllowedOperation('reboot',$cleanly);
        if( !$allowed['allowed'] ) throw new Exception($allowed['cause']);

        $vmid = $this->getId();
        $ref = $this->cluster->getRefByUuid('VM',$vmid);

        if( !$cleanly ){
            $data = $this->cluster->getDriver()->get('Async.VM.hard_reboot',array($ref));
        }
        else{
            $data = $this->cluster->getDriver()->get('Async.VM.clean_reboot',array($ref));
        }*/
    }

    public function isAllowedOperation($key,$mode=NULL){
        /*
        $operation = $this->operationToString($key,$mode);
        $op_arr = explode('_',$operation);

        if($op_arr[0] == 'migrate' ){
            $ret['allowed'] = FALSE;
            $ret['cause'] = 'Operation not available for XenServer driver';
            return $ret;
        }
        if($operation == 'snapshot_withram' ){
            $ret['allowed'] = FALSE;
            $ret['cause'] = 'Operation not available for XenServer driver';
            return $ret;
        }
        return parent::isAllowedOperation($operation,NULL);
        */
    }

    public function migrate($target,$online){
        /*
        throw new Exception('Not implemented');

        $allowed = $this->isAllowedOperation('migrate',$online);
        if( !$allowed['allowed'] ) throw new Exception($allowed['cause']);
        //Not implemented for Xenserver Driver. Is available into XAPI API but not documented.
        */
    }

    public function cloneVM($target_name,$full=TRUE){
        /*$allowed = $this->isAllowedOperation('clone');
        if( !$allowed['allowed'] ) throw new Exception($allowed['cause']);
        $this->getCluster()->cloneVM($this,$target_name,TRUE);*/
    }

    public function createSnapshot($name,$description='',$withRam=FALSE){
/*        $allowed = $this->isAllowedOperation('snapshot',$withRam);
        if( !$allowed['allowed'] ) throw new Exception($allowed['cause']);

        $vmid = $this->getId();
        $ref = $this->cluster->getRefByUuid('VM',$vmid);

        $refsnap=$this->cluster->getDriver()->get('VM.snapshot',array($ref,$name));
        $this->cluster->getDriver()->get('VM.set_name_description',array($refsnap,$description));*/
    }

    public function reverseToSnapshot($snapid){/*
        $allowed = $this->isAllowedOperation('snapshot_reverse');
        if( !$allowed['allowed'] ) throw new Exception($allowed['cause']);

        $ref = $this->cluster->getRefByUuid('VM',$snapid);
        $this->cluster->getDriver()->get('VM.revert',array($ref));*/
    }

    public function removeSnapshot($snapid){
    /*    $allowed = $this->isAllowedOperation('snapshot_remove');
        if( !$allowed['allowed'] ) throw new Exception($allowed['cause']);

        $ref = $this->cluster->getRefByUuid('VM',$snapid);
        $this->cluster->getDriver()->get('VM.destroy',array($ref));*/
    }
}
