<?php
include_once 'common/HttpClient.php';

/**
    @file
    @brief Permet simplifier l'envoi de requête vers une API Xenserver XAPI
    @details
		La classe XenXapiClient de ce fichier étends la classe HTTPClient en redéfinissant
		certaines méthodes pour simplifier une utilisation avec XAPI de XenServer.

		Cette classe ne gére par les RRDs, dans Xenserver les RRDs sont récupérés via
		une autre API. Voir XenRRDClient.

		Pour utiliser ce fichier, cette classe, il est nécessaire d'avoir la librairie
		php xmlrpc installée. Package php-xmlrpc soud Debian.
    @author Julien George
*/

class XenXapiClient extends HttpClient
{
	protected $driverSettings;
	protected $session = NULL;
	protected $rrd_client = NULL;

	public function __construct($settings)
	{
		$this->driverSettings = $settings;
		if( isset( $settings['url'] ) ){
			$url = $settings['url'];
			$parse = parse_url($settings['url']);
			$keys = array('scheme','host','port');
			foreach($keys as $key){
				$this->driverSettings[$key] = $parse[$key];
			}
		}
		else
		{
			$scheme = @$this->driverSettings['scheme'] ? : 'https';
			$fqdn = @$this->driverSettings['host'];
			$port = @$this->driverSettings['port'] ? : 443;
			$url = "$scheme://$fqdn:$port";
		}

        	$path_prefix = '';

		parent::__construct("$url/$path_prefix");

		$header[] = "Content-type: text/xml";
		$this->setoptions(CURLOPT_HTTPHEADER, $header);
        	$this->setoptions(CURLOPT_TIMEOUT, 20);
		$this->setoptions(CURLOPT_CONNECTTIMEOUT, 20);
		$this->setCookies(FALSE);
	}

	public function open()
	{
		parent::open();
		$params[] = @$this->driverSettings['login'] ? : 'root';
		$params[] = @$this->driverSettings['password'];

		$ticketfile =  sys_get_temp_dir() .'/xen_'. md5($this->driverSettings['host'].$params[0].$params[1]);
		$tf_content = @file_get_contents($ticketfile);
		if( $tf_content == FALSE){
		        $data = $this->get('session.login_with_password',$params);
		        file_put_contents($ticketfile,$data);
		}
		else{
		        $data = $tf_content;
		}

		$this->session = $data;
	}

	public function get($object,$params=NULL,$xml_rpc=TRUE){
		if( $params == NULL ) $params=array();
                else if(is_string($params)) $params=explode(',',$params);

		if( $this->session != NULL ) array_unshift($params,$this->session);

                $request = xmlrpc_encode_request($object,$params);
                $header[] = "Content-type: text/xml";
                $header[] = "Content-length: ".strlen($request);

		$options[CURLOPT_POSTFIELDS] = $request;
		$options[CURLOPT_HTTPHEADER] = $header;
		$options[CURLOPT_URL] = $this->url;

		$data = $this->execute_request($options);
		               if ($this->getHttpCode() != 200)
                        throw new Exception('Http error : '.$this->getHttpCode());

                $result = xmlrpc_decode($data);

                if(isset($result["ErrorDescription"])) throw new SessionException($result);
                else if(isset($result["Value"])) return $result["Value"];
                else {
                        echo "UNKNOWN ERROR :";
                        echo "<pre>";
                        print_r($result);
                        echo "</pre>";
                }
	}

	public function close(){
//		$this->get("session.logout");
		$this->session=NULL;
                parent::close();
	}
}

class SessionException extends Exception{
	private $type;
	private $value;
	private $param;
	private $errorDescription;

	private static $types=array(-1=>"UNKNOWN","MESSAGE_METHOD_UNKNOWN","SESSION_INVALID","UUID_INVALID","MESSAGE_PARAMETER_COUNT_MISMATCH","FIELD_TYPE_ERROR");

	public static function toCode($type){
		$codes=array_flip(Self::$types);
		if(isset( $codes[$type])) return $codes[$type];
		else return -1;
	}

	public static function toType($code){
                return $this->types[$code];
	}

	public function __construct($result){
		$this->errorDescription = $result["ErrorDescription"];
		$this->type=$result["ErrorDescription"][0];
                $code=self::toCode($this->type);


                $msg="Type : $this->type";

		if(isset($result["ErrorDescription"][1])&& strlen($result["ErrorDescription"][1]) >0){
                	$this->value=$result["ErrorDescription"][1];
			$msg="$msg for element $this->value";
		}

		if(isset($result["ErrorDescription"][2]) && strlen($result["ErrorDescription"][2]) >0){
			$this->param=$result["ErrorDescription"][2];
			$msg="$msg : $this->param";
		}

                parent::__construct($msg,$code);
	}

	public function getType(){
		return $this->type;
	}

	public function getValue(){
	 	return $this->value;
	}

	public function getParam(){
		return $this->param;
	}

	public function getErrorDescription(){
		return $this->errorDescription;
	}
}
