<?php
include_once 'common/HttpClient.php';

/**
    @file
    @brief Permet simplifier l'envoi de requête vers l'API RRDs de XenServer
    @details
		La classe XenRRDClient de ce fichier étends la classe HTTPClient en redéfinissant
		certaines méthodes pour simplifier une utilisation avec XAPI de XenServer.

		Cette classe ne gére que les RRDs. Pour d'autres données, il est nécessaire
		d'utiliser la classe XapiClient contenue dans le fichier du même nom.
    @author Julien George
*/

class XenRRDClient extends HttpClient
{
	protected $driverSettings;
	protected $address = NULL;

	public function __construct($settings)
	{
		$this->driverSettings = $settings;
		if( isset( $settings['url'] ) ){
			$url = $settings['url'];
			$parse = parse_url($settings['url']);
			$keys = array('scheme','host','port');
			foreach($keys as $key){
				$this->driverSettings[$key] = $parse[$key];
			}
		}

     	$scheme = @$this->driverSettings['scheme'] ? : 'https';
       	$fqdn = @$this->driverSettings['host'];
       	$port = @$this->driverSettings['port'] ? : 8006;

		$user = @$this->driverSettings['login'] ? : 'root';
        $pass = @$this->driverSettings['password'];

        $path_prefix = '';

		$url = "$scheme://$user:$pass@$fqdn:$port/$path_prefix";
		parent::__construct($url);

		$this->setCookies(FALSE);
		$this->setAddress($fqdn);
	}

	public function setAddress($address){
		$old = $this->address;
		$this->url = str_replace("@$old:","@$address:",$this->url);
		$this->address = $address;
	}

	public function get($path,$params=NULL,$p=NULL){
		$data = parent::get($path,$params);
		 if ( $this->getHttpCode() != 200)
                        throw new Exception('Http error : '.$this->getHttpCode());

                $xml = simplexml_load_string($data, "SimpleXMLElement", LIBXML_NOCDATA);
                $json = json_encode($xml);
                $data = json_decode($json,TRUE);

		return $data;
	}

	public function get_rrd($host=TRUE,$period=60){
		$params['start'] = time()-$period;
		if( $host ) $params['host']='true';
		$data = $this->get('rrd_updates',$params);
		$vm_entries=array();
                foreach($data['meta']['legend']['entry'] as $idx => $entry){
                        $exp = explode(':',$entry);
                        if( $exp[0] == 'AVERAGE' ){
                                $entry_type = $exp[1];
                                $entry_uuid = $exp[2];
                                $entry_name = $exp[3];
				$i=0;
				$max = 0;
				while(isset($data['data']['row'][$i]['v'][$idx])){
                                	$entry_value = $data['data']['row'][$i]['v'][$idx];
					if( $entry_value > $max ) $max = $entry_value;
					$i++;
				}
                                $vm_entries[$entry_type][$entry_uuid][$entry_name] = $max;
                        }
                }
		return $vm_entries;
	}

	public function vm_rrd(){
		return $this->get_rrd(FALSE)['vm'];
	}
	public function host_rrd(){
		return $this->get_rrd(TRUE)['host'];
	}
}
